#ifndef VPD_UTIL_H
#define VPD_UTIL_H

/*
  util - VPD utilitiy functions.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: util.h,v 1.12 2004/01/08 03:53:33 martins Exp $
*/
  
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <sys/types.h>

extern char * vpd_program_name;

extern void
die(const char *  /* message */
	);

extern void
die2(const char *,  /* message */
     const char *   /* sub-message */
	);

extern char *
build_relative_filename(const char *, /* dir */
			const char *  /* relpath */
	);

extern bool
file_read_contents(const char  *, /* pathname */
		   char **,       /* buf      */
		   size_t *,      /* length   */
		   int  *         /* err      */
	);

extern void
file_relative_read_contents(const char  *, /* dir      */
			    const char  *, /* relpath  */
			    char **,       /* buf      */
			    size_t *       /* length    */
	);

extern bool
file_write_contents(const  char  *, /* filename */
		    const  char  *, /* buf      */
		    size_t,         /* count    */
		    int *           /* err     */
	);

extern void
file_relative_write_contents(const char *, /* dir      */
			     const char *, /* relpath  */
			     const char *, /* data     */
			     size_t        /* count    */
	);

/* Some of the shifts and masks below are redundant, but that's why we
 * have an optimiser!  Isn't it nice?  :-)
 */

static inline uint16_t
le16_to_cpu(uint16_t n)
{
	return 	((uint16_t)(((uint8_t*)&n)[1]) << 8) |
		((uint16_t)(((uint8_t*)&n)[0]) << 0) ;
}

static inline uint16_t
cpu_to_le16(uint16_t n)
{
	uint16_t ret;
	uint8_t * t = (uint8_t *) &ret;

	t[0] = (n >> 0) & 0xff;
	t[1] = (n >> 8) & 0xff;

	return ret;
}

static inline uint32_t
le32_to_cpu(uint32_t n)
{
	return 	((uint32_t)(((uint8_t*)&n)[3]) << 24) |
		((uint32_t)(((uint8_t*)&n)[2]) << 16) |
		((uint32_t)(((uint8_t*)&n)[1]) <<  8) |
		((uint32_t)(((uint8_t*)&n)[0]) <<  0) ;
}

static inline uint32_t
cpu_to_le32(uint32_t n)
{
	uint32_t ret;
	uint8_t * t = (uint8_t *) &ret;

	t[0] = (n >>  0) & 0xff;
	t[1] = (n >>  8) & 0xff;
	t[2] = (n >> 16) & 0xff;
	t[3] = (n >> 24) & 0xff;

	return ret;
}

extern ssize_t
big_cool_pread(int fd, uint8_t *buf, size_t count, off_t offset);

extern ssize_t
big_cool_pwrite(int fd, const uint8_t *buf, size_t count, off_t offset);

/* string_array is useful for making sparse, fixed arrays without
 * needing to do bounds checking. */

struct string_array {
	int key;
	const char * val;
};

extern const char *
string_array_lookup(const struct string_array * a,
		    const char * d,
		    int k);

#endif
